/* SPDX-FileCopyrightText: 2011, 2014 - Jesse van den Kieboom
 * SPDX-FileCopyrightText: 2015 - Sébastien Wilmet
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#pragma once

#include <glib.h>

G_BEGIN_DECLS

G_GNUC_INTERNAL
gchar *	_gspell_osx_get_preferred_spell_language	(void);

G_END_DECLS
